@@ -4,6 +4,8 @@ __all__ = ["aiopen", "aiexists", "ailistdir"]
 
 import contextlib
 import io
+from collections.abc import AsyncIterator
+from types import TracebackType
 
 try:
     import aiofiles.os
@@ -15,26 +17,36 @@ except ImportError:
     import os
 
     class AsyncFile:
+        """Async file wrapper."""
+
         def __init__(self, file: io.TextIOWrapper):
             self.file = file
 
         async def readlines(self) -> list[str]:
+            """Read lines."""
             return self.file.readlines()
 
         async def __aenter__(self):
             return self
 
-        async def __aexit__(self, exc_type, exc_val, exc_tb):
+        async def __aexit__(
+            self,
+            exc_type: type[BaseException] | None,
+            exc_val: BaseException | None,
+            exc_tb: TracebackType | None,
+        ) -> None:
             self.file.close()
 
-    @contextlib.asynccontextmanager
-    async def aiopen(*args, **kwargs) -> AsyncFile:
-        yield AsyncFile(open(*args, **kwargs))
+    @contextlib.asynccontextmanager  # type: ignore[no-redef]
+    async def aiopen(*args, **kwargs) -> AsyncIterator[AsyncFile]:
+        """Async > sync wrapper."""
+        with open(*args, **kwargs) as f:  # noqa: ASYNC101, ASYNC230, pylint: disable=unspecified-encoding
+            yield AsyncFile(f)
 
     async def aiexists(*args, **kwargs) -> bool:
         """Async > sync wrapper."""
         return os.path.exists(*args, **kwargs)
 
-    async def ailistdir(*args, **kwargs) -> list[str]:
+    async def ailistdir(*args, **kwargs) -> list[str]:  # type: ignore[no-redef]
         """Async > sync wrapper."""
         return os.listdir(*args, **kwargs)
