@@ -1,6 +1,7 @@
 """Pyprland - an Hyprland companion app (cli client & daemon)."""
 
 import asyncio
+import contextlib
 import importlib
 import itertools
 import json
@@ -38,7 +39,7 @@ def remove_duplicate(names: list[str]) -> Callable:
     Will check arguments as well
     """
 
-    def _remove_duplicates(fn: Callable) -> Callable:
+    def _remove_duplicates(func: Callable) -> Callable:
         """Wrapper for the decorator."""
 
         async def _wrapper(self: "Pyprland", full_name: str, *args, **kwargs) -> bool:
@@ -48,7 +49,7 @@ def remove_duplicate(names: list[str]) -> Callable:
                 if key == _dedup_last_call.get(full_name):
                     return True
                 _dedup_last_call[full_name] = key
-            return cast("bool", await fn(self, full_name, *args, **kwargs))
+            return cast("bool", await func(self, full_name, *args, **kwargs))
 
         return _wrapper
 
@@ -173,6 +174,15 @@ class Pyprland:
 
         init_pyprland = "pyprland" not in self.plugins
 
+        current_plugins = set(self.plugins.keys())
+        new_plugins = set(["pyprland"] + self.config["pyprland"]["plugins"])
+        for name in current_plugins - new_plugins:
+            self.log.info("Unloading plugin %s", name)
+            plugin = self.plugins.pop(name)
+            await plugin.exit()
+            if name in self.queues:
+                await self.queues.pop(name).put(None)
+
         for name in ["pyprland"] + self.config["pyprland"]["plugins"]:
             if name not in self.plugins and not await self._load_single_plugin(name, init):
                 continue
@@ -266,6 +276,7 @@ class Pyprland:
         await asyncio.wait_for(asyncio.gather(*active_plugins), timeout=TASK_TIMEOUT / 2)
 
     async def _abort_plugins(self, writer: asyncio.StreamWriter) -> None:
+        """Abort all plugins and stop the server."""
         await self.exit_plugins()
         # cancel the task group
         for task in self.tasks:
@@ -315,13 +326,14 @@ class Pyprland:
             full_name = f"run_{cmd}"
 
             if PYPR_DEMO:
-                os.system(f"notify-send -t 4000 '{data}'")  # noqa: ASYNC221
+                os.system(f"notify-send -t 4000 '{data}'")  # noqa: ASYNC221, ASYNC102
 
             if not await self._call_handler(full_name, *args, notify=cmd):
                 self.log.warning("No such command: %s", cmd)
 
         writer.write(b"\n")
-        await writer.drain()
+        with contextlib.suppress(BrokenPipeError, ConnectionResetError):
+            await writer.drain()
         writer.close()
 
     async def serve(self) -> None:
@@ -420,6 +432,7 @@ async def run_daemon() -> None:
 
 
 def get_commands_help(manager: Pyprland) -> dict:
+    """Get the available commands and their documentation."""
     docs = {
         "edit": "Edit the configuration file. (not in pypr-client)",
         "dumpjson": "Dump the configuration in JSON format.",
