@@ -3,17 +3,17 @@
 import asyncio
 import contextlib
 import time
-from dataclasses import dataclass
-from enum import Flag, auto
+from collections.abc import Iterable
 from functools import partial
-from typing import TYPE_CHECKING, cast
+from typing import cast
 
 from ...adapters.units import convert_coords, convert_monitor_dimension
-from ...common import MINIMUM_ADDR_LEN, CastBoolMixin, apply_variables, is_rotated, state
+from ...common import MINIMUM_ADDR_LEN, apply_variables, is_rotated, state
 from ...ipc import get_client_props, get_monitor_props, notify_error
 from ...types import ClientInfo, MonitorInfo, VersionInfo
 from ..interface import Plugin
 from .animations import AnimationTarget, Placement
+from .common import FocusTracker, HideFlavors
 from .helpers import (
     apply_offset,
     compute_offset,
@@ -25,9 +25,6 @@ from .helpers import (
 from .lookup import ScratchDB
 from .objects import Scratch
 
-if TYPE_CHECKING:
-    from collections.abc import Callable
-
 AFTER_SHOW_INHIBITION = 0.3  # 300ms of ignorance after a show
 DEFAULT_MARGIN = 60  # in pixels
 DEFAULT_HIDE_DELAY = 0  # in seconds
@@ -35,47 +32,58 @@ DEFAULT_HYSTERESIS = 0.4  # in seconds
 
 
 # Ad-hoc classes & functions {{{
-class HideFlavors(Flag):
-    """Flags for different hide behavior."""
+class WindowRuleSet:
+    """Windowrule set builder."""
 
-    NONE = auto()
-    FORCED = auto()
-    TRIGGERED_BY_AUTOHIDE = auto()
-    IGNORE_TILED = auto()
+    def __init__(self) -> None:
+        self._params: list[tuple[str, str]] = []
+        self._class = ""
+        self._name = "PyprScratchR"
 
+    def set_class(self, value: str) -> None:
+        """Set the windowrule matching class."""
+        self._class = value
 
-@dataclass
-class FocusTracker:
-    """Focus tracking object."""
+    def set_name(self, value: str) -> None:
+        """Set the windowrule name."""
+        self._name = value
 
-    prev_focused_window: str
-    prev_focused_window_wrkspc: str
+    def set(self, param: str, value: str) -> None:
+        """Set a windowrule property."""
+        self._params.append((param, value))
 
-    def clear(self) -> None:
-        """Clear the tracking."""
-        self.prev_focused_window = ""
-        self.prev_focused_window_wrkspc = ""
+    def _get_content(self) -> Iterable[str]:
+        """Get the windowrule content."""
+        if state.hyprland_version > VersionInfo(0, 47, 2):
+            if state.hyprland_version < VersionInfo(0, 53, 0):
+                for p in self._params:
+                    yield f"windowrule {p[0]} {p[1]}, class: {self._class}"
+            elif self._name:
+                yield f"windowrule[{self._name}]:enable true"
+                yield f"windowrule[{self._name}]:match:class {self._class}"
+                for p in self._params:
+                    yield f"windowrule[{self._name}]:{p[0]} {p[1]}"
+            else:
+                for p in self._params:
+                    yield f"windowrule {p[0]} {p[1]}, match:class {self._class}"
+        else:
+            for p in self._params:
+                yield f"windowrule {p[0]} {p[1]}, ^({self._class})$"
+
+    def get_content(self) -> list[str]:
+        """Get the windowrule content."""
+        return list(self._get_content())
 
 
 def get_animation_type(scratch: Scratch) -> str:
     """Get the animation type or an empty string if not set."""
-    return scratch.conf.get("animation", "").lower()
-
-
-def class_decorator_old(name: str) -> str:
-    """Return the class rule for old hyprland versions."""
-    return f"^({name})$"
-
-
-def class_decorator_new(name: str) -> str:
-    """Return the class rule for new hyprland versions."""
-    return f"class:{name}"
+    return scratch.conf.get_str("animation", "").lower()
 
 
 # }}}
 
 
-class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstring {{{
+class Extension(Plugin):  # pylint: disable=missing-class-docstring {{{
     """Scratchpads addon."""
 
     procs: dict[str, asyncio.subprocess.Process] = {}  # pylint: disable=no-member
@@ -92,7 +100,6 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
     def __init__(self, name: str) -> None:
         super().__init__(name)
         self._hysteresis_tasks = {}
-        self._classify: Callable
         self.get_client_props = staticmethod(partial(get_client_props, logger=self.log))
         Scratch.get_client_props = self.get_client_props
         self.get_monitor_props = staticmethod(partial(get_monitor_props, logger=self.log))
@@ -122,7 +129,6 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
 
     async def on_reload(self) -> None:
         """Config loader."""
-        self._classify = class_decorator_new if state.hyprland_version > VersionInfo(0, 47, 2) else class_decorator_old
         # Sanity checks
         _scratch_classes: dict[str, str] = {}
         for uid, scratch in self.config.items():
@@ -140,7 +146,7 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
                 _scratch_classes[_klass] = uid
 
         # Create new scratches with fresh config items
-        scratches = {name: Scratch(name, self.config) for name, options in self.config.items()}
+        scratches = {name: Scratch(name, self.config) for name, options in self.config.iter_subsections()}
 
         scratches_to_spawn = set()
         for name, new_scratch in scratches.items():
@@ -150,7 +156,7 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
             else:
                 # else register it
                 self.scratches.register(new_scratch, name)
-                is_lazy = self.cast_bool(new_scratch.conf.get("lazy"), False)
+                is_lazy = new_scratch.conf.get_bool("lazy", True)
                 if not is_lazy:
                     scratches_to_spawn.add(name)
 
@@ -170,29 +176,39 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
         """Unset the windowrules."""
         defined_class = scratch.conf.get("class", "")
         if defined_class:
-            await self.hyprctl(f"windowrule unset,{self._classify(defined_class)}", "keyword")
+            if state.hyprland_version < VersionInfo(0, 53, 0):
+                if state.hyprland_version > VersionInfo(0, 47, 2):
+                    await self.hyprctl(f"windowrule unset, class: {defined_class}", "keyword")
+                else:
+                    await self.hyprctl(f"windowrule unset, ^({defined_class})$", "keyword")
+            else:
+                await self.hyprctl(f"windowrule[{scratch.uid}]:enable false", "keyword")
 
     async def _configure_windowrules(self, scratch: Scratch) -> None:
         """Set initial client window state (sets windowrules)."""
         self.scratches.set_state(scratch, "configured")
-        animation_type: str = cast("str", scratch.conf.get("animation", "fromTop")).lower()
-        defined_class: str = cast("str", scratch.conf.get("class", ""))
+        animation_type: str = scratch.conf.get_str("animation", "fromTop").lower()
+        defined_class: str = scratch.conf.get_str("class", "")
         skipped_windowrules: list[str] = cast("list", scratch.conf.get("skip_windowrules", []))
+        wr = WindowRuleSet()
+        wr.set_class(defined_class)
+        wr.set_name(scratch.uid)
         if defined_class:
             forced_monitor = scratch.conf.get("force_monitor")
             if forced_monitor and forced_monitor not in state.monitors:
                 self.log.error("forced monitor %s doesn't exist", forced_monitor)
                 await self.notify_error(f"Monitor '{forced_monitor}' doesn't exist, check {scratch.uid}'s scratch configuration")
                 forced_monitor = None
-            monitor = await self.get_monitor_props(name=forced_monitor)
-            width, height = convert_coords(cast("str", scratch.conf.get("size", "80% 80%")), monitor)
-
-            ipc_commands = []
+            monitor = await self.get_monitor_props(name=cast("str | None", forced_monitor))
+            width, height = convert_coords(scratch.conf.get_str("size", "80% 80%"), monitor)
 
             if "float" not in skipped_windowrules:
-                ipc_commands.append(f"windowrule float,{self._classify(defined_class)}")
+                if state.hyprland_version < VersionInfo(0, 53, 0):
+                    wr.set("float", "")
+                else:
+                    wr.set("float", "on")
             if "workspace" not in skipped_windowrules:
-                ipc_commands.append(f"windowrule workspace {mk_scratch_name(scratch.uid)} silent,{self._classify(defined_class)}")
+                wr.set("workspace", f"{mk_scratch_name(scratch.uid)} silent")
             set_aspect = "aspect" not in skipped_windowrules
 
             if animation_type:
@@ -209,12 +225,12 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
                     "fromleft": f"-200% {margin_y}",
                 }[animation_type]
                 if set_aspect:
-                    ipc_commands.append(f"windowrule move {t_pos},{self._classify(defined_class)}")
+                    wr.set("move", t_pos)
 
             if set_aspect:
-                ipc_commands.append(f"windowrule size {width} {height},{self._classify(defined_class)}")
+                wr.set("size", f"{width} {height}")
 
-            await self.hyprctl(ipc_commands, "keyword")
+            await self.hyprctl(wr.get_content(), "keyword")
 
     async def __wait_for_client(self, scratch: Scratch, use_proc: bool = True) -> bool:
         """Wait for a client to be up and running.
@@ -292,7 +308,7 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
         if not item.have_command:
             return True
 
-        if self.cast_bool(cast("bool", item.conf.get("process_tracking")), True):
+        if item.conf.get_bool("process_tracking", True):
             if not await item.is_alive():
                 await self._configure_windowrules(item)
                 self.log.info("%s is not running, starting...", uid)
@@ -310,7 +326,7 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
         assert scratch
         self.scratches.set_state(scratch, "respawned")
         old_pid = self.procs[name].pid if name in self.procs else 0
-        command = apply_variables(scratch.conf["command"], state.variables)
+        command = apply_variables(cast("str", scratch.conf["command"]), state.variables)
         proc = await asyncio.create_subprocess_shell(command)
         self.procs[name] = proc
         pid = proc.pid
@@ -403,7 +419,7 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
 
     async def _hysteresis_handling(self, scratch: Scratch) -> None:
         """Hysteresis handling."""
-        hysteresis = scratch.conf.get("hysteresis", DEFAULT_HYSTERESIS)
+        hysteresis = scratch.conf.get_float("hysteresis", DEFAULT_HYSTERESIS)
         if hysteresis:
             self.cancel_task(scratch.uid)
 
@@ -495,7 +511,7 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
             await self.hyprctl(f"pin address:{focused}")
 
     async def run_toggle(self, uid_or_uids: str) -> None:
-        """<name> toggles visibility of scratchpad "name"."""
+        """<name> toggles visibility of scratchpad "name" (supports multiple names)."""
         uids = list(filter(bool, map(str.strip, uid_or_uids.split()))) if " " in uid_or_uids else [uid_or_uids.strip()]
 
         for uid in uids:
@@ -513,7 +529,7 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
             first_scratch.meta.space_identifier,
             get_active_space_identifier(),
         )
-        if self.cast_bool(first_scratch.conf.get("alt_toggle")):
+        if first_scratch.conf.get_bool("alt_toggle"):
             # Needs to be on any monitor (if workspace matches)
             extra_visibility_check = first_scratch.meta.space_identifier in await get_all_space_identifiers(
                 await self.hyprctl_json("monitors")
@@ -543,14 +559,14 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
 
     async def get_offsets(self, scratch: Scratch, monitor: MonitorInfo | None = None) -> tuple[int, int]:
         """Return offset from config or use margin as a ref."""
-        offset: int | str = scratch.conf.get("offset")
+        offset = scratch.conf.get("offset")
         if monitor is None:
             monitor = await get_monitor_props(self.log, name=scratch.forced_monitor)
         rotated = is_rotated(monitor)
         aspect = reversed(scratch.client_info["size"]) if rotated else scratch.client_info["size"]
 
         if offset:
-            return cast("tuple[int, int]", (convert_monitor_dimension(offset, ref, monitor) for ref in aspect))
+            return cast("tuple[int, int]", (convert_monitor_dimension(cast("str", offset), ref, monitor) for ref in aspect))
 
         mon_size = [monitor["height"], monitor["width"]] if rotated else [monitor["width"], monitor["height"]]
 
@@ -565,7 +581,7 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
             return False
 
         await self._slide_animation(animation_type, scratch, await self.get_offsets(scratch, monitor))
-        delay: float = scratch.conf.get("hide_delay", DEFAULT_HIDE_DELAY)
+        delay: float = scratch.conf.get_float("hide_delay", DEFAULT_HIDE_DELAY)
         if delay:
             await asyncio.sleep(delay)  # await for animation to finish
         return True
@@ -596,7 +612,7 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
         )
 
     async def run_show(self, uid: str) -> None:
-        """<name> shows scratchpad "name"."""
+        """<name> shows scratchpad "name" (accepts "*")."""
         if uid == "*":
             await asyncio.gather(*(self.run_show(s.uid) for s in self.scratches.values() if not s.visible))
             return
@@ -616,10 +632,13 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
             return
 
         excluded_ids = scratch.conf.get("excludes", [])
-        restore_excluded = self.cast_bool(scratch.conf.get("restore_excluded", False))
+        restore_excluded = scratch.conf.get_bool("restore_excluded", False)
         if excluded_ids == "*":
             excluded_ids = [excluded.uid for excluded in self.scratches.values() if excluded.uid != scratch.uid]
-        for e_uid in excluded_ids:
+        elif excluded_ids is None:
+            excluded_ids = []
+
+        for e_uid in cast("list[str]", excluded_ids):
             excluded = self.scratches.get(e_uid)
             assert excluded
             if excluded.visible:
@@ -641,9 +660,12 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
 
     async def _handle_multiwindow(self, scratch: Scratch, clients: list[ClientInfo]) -> bool:
         """Collect every matching client for the scratchpad and add them to extra_addr if needed."""
-        if not self.cast_bool(scratch.conf.get("multi"), True):
+        if not scratch.conf.get_bool("multi", True):
+            return False
+        try:
+            match_by, match_value = scratch.get_match_props()
+        except KeyError:
             return False
-        match_by, match_value = scratch.get_match_props()
         match_fn = get_match_fn(match_by, match_value)
         hit = False
         for client in clients:
@@ -653,12 +675,14 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
                 address = client["address"]
                 if address not in scratch.extra_addr:
                     scratch.extra_addr.add(address)
+                    if scratch.conf.get("pinned", True):
+                        await self.hyprctl(f"pin address:{address}")
                     hit = True
         return hit
 
     async def _show_transition(self, scratch: Scratch, monitor: MonitorInfo, was_alive: bool) -> None:
         """Performs the transition to visible state."""
-        forbid_special = not self.cast_bool(scratch.conf.get("allow_special_workspace"), True)
+        forbid_special = not scratch.conf.get_bool("allow_special_workspace", True)
         wrkspc = (
             monitor["activeWorkspace"]["name"]
             if forbid_special
@@ -671,7 +695,7 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
 
         scratch.meta.last_shown = time.time()
         # Start the transition
-        preserve_aspect = self.cast_bool(scratch.conf.get("preserve_aspect"))
+        preserve_aspect = scratch.conf.get_bool("preserve_aspect")
         should_set_aspect = (
             not (preserve_aspect and was_alive) or scratch.monitor != state.active_monitor
         )  # Not aspect preserving or it's newly spawned
@@ -743,7 +767,7 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
     async def _animate_show(self, scratch: Scratch, monitor: MonitorInfo, relative_animation: bool) -> None:
         """Animate the show transition."""
         animation_type = get_animation_type(scratch)
-        multiwin_enabled = self.cast_bool(scratch.conf.get("multi"), True)
+        multiwin_enabled = scratch.conf.get_bool("multi", True)
         if animation_type:
             animation_commands = []
 
@@ -757,7 +781,7 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
                     animation_type,
                     monitor,
                     scratch.client_info,
-                    scratch.conf.get("margin", DEFAULT_MARGIN),
+                    scratch.conf.get_int("margin", DEFAULT_MARGIN),
                 )
             animation_commands.append(list(main_win_position) + [scratch.full_address])
 
@@ -774,10 +798,10 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
         """Apply the size from config."""
         size = scratch.conf.get("size")
         if size:
-            width, height = convert_coords(size, monitor)
+            width, height = convert_coords(cast("str", size), monitor)
             max_size = scratch.conf.get("max_size")
             if max_size:
-                max_width, max_height = convert_coords(max_size, monitor)
+                max_width, max_height = convert_coords(cast("str", max_size), monitor)
                 width = min(max_width, width)
                 height = min(max_height, height)
             await self.hyprctl(f"resizewindowpixel exact {width} {height},address:{scratch.full_address}")
@@ -786,14 +810,14 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
         """Apply the `position` config parameter."""
         position = scratch.conf.get("position")
         if position:
-            x_pos, y_pos = convert_coords(position, monitor)
+            x_pos, y_pos = convert_coords(cast("str", position), monitor)
             x_pos_abs, y_pos_abs = x_pos + monitor["x"], y_pos + monitor["y"]
             await self.hyprctl(f"movewindowpixel exact {x_pos_abs} {y_pos_abs},address:{scratch.full_address}")
             return True
         return False
 
-    async def run_hide(self, uid: str, flavor: HideFlavors = HideFlavors.NONE) -> None:  # noqa: PLR0912, C901
-        """<name> hides scratchpad "name"."""
+    async def run_hide(self, uid: str, flavor: HideFlavors = HideFlavors.NONE) -> None:  # noqa: C901
+        """<name> hides scratchpad "name" (accepts "*")."""
         if uid == "*":
             await asyncio.gather(*(self.run_hide(s.uid) for s in self.scratches.values() if s.visible))
             return
@@ -819,13 +843,17 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
             self.log.warning("%s is already hidden", uid)
             return
 
+        await self._hide_scratch(scratch, active_window, active_workspace)
+
+    async def _hide_scratch(self, scratch: Scratch, active_window: str, active_workspace: str) -> None:
+        """Perform the actual hide operation."""
         clients = await self.hyprctl_json("clients")
         await scratch.update_client_info(clients=clients)
         ref_position = scratch.client_info["at"]
         monitor_info = scratch.meta.monitor_info
         scratch.meta.extra_positions[scratch.address] = compute_offset(ref_position, (monitor_info["x"], monitor_info["y"]))
         # collects window which have been created by the app
-        if self.cast_bool(scratch.conf.get("multi"), True):
+        if scratch.conf.get_bool("multi", True):
             await self._handle_multiwindow(scratch, clients)
             positions = {}
             for sub_client in clients:
@@ -834,15 +862,15 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
             scratch.meta.extra_positions.update(positions)
         scratch.visible = False
         scratch.meta.should_hide = False
-        self.log.info("Hiding %s", uid)
+        self.log.info("Hiding %s", scratch.uid)
         await self._pin_scratch(scratch)
         await self._hide_transition(scratch, monitor_info)
 
-        if not self.cast_bool(scratch.conf.get("close_on_hide"), False):
-            await self.hyprctl(f"movetoworkspacesilent {mk_scratch_name(uid)},address:{scratch.full_address}")
+        if not scratch.conf.get_bool("close_on_hide", False):
+            await self.hyprctl(f"movetoworkspacesilent {mk_scratch_name(scratch.uid)},address:{scratch.full_address}")
 
             for addr in scratch.extra_addr:
-                await self.hyprctl(f"movetoworkspacesilent {mk_scratch_name(uid)},address:{addr}")
+                await self.hyprctl(f"movetoworkspacesilent {mk_scratch_name(scratch.uid)},address:{addr}")
                 await asyncio.sleep(0.01)
         else:
             await self.hyprctl(f"closewindow address:{scratch.full_address}")
@@ -858,7 +886,7 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
 
     async def _handle_focus_tracking(self, scratch: Scratch, active_window: str, active_workspace: str, clients: ClientInfo | dict) -> None:
         """Handle focus tracking."""
-        if not self.cast_bool(scratch.conf.get("smart_focus"), True):
+        if not scratch.conf.get_bool("smart_focus", True):
             return
         for track in self.focused_window_tracking.values():
             if scratch.have_address(track.prev_focused_window):
@@ -866,7 +894,8 @@ class Extension(CastBoolMixin, Plugin):  # pylint: disable=missing-class-docstri
         tracker = self.focused_window_tracking.get(scratch.uid)
         if tracker and not tracker.prev_focused_window_wrkspc.startswith("special:"):
             same_workspace = tracker.prev_focused_window_wrkspc == active_workspace
-            client = next(filter(lambda d: d.get("address") == tracker.prev_focused_window, clients), None)
+            t_pfw = tracker.prev_focused_window
+            client = next(filter(lambda d: d.get("address") == t_pfw, cast("list[dict]", clients)), None)
             if (
                 client
                 and scratch.have_address(active_window)
